#include <Arduino.h>
#include "Automaton.h"
#include "TeensyStep.h"

bool debug = true ;

//stepper motor
int enablePin = 22 ;
Stepper motor(15, 14);      
RotateControl controller;  
bool stepper_mode = false ;

int MSTEPPING = 16 ;

int stepper_maxSpeed = 900*MSTEPPING ;
int stepper_maxAcc = 1200*MSTEPPING;
int current_speed ;

int STEPS_PER_TURN = 200 ;
float MOTOR_RATIO = 99.05 ;
float BELT_RATIO = 5.217391304 ;
float STEPS_2_ROT = STEPS_PER_TURN * MSTEPPING * MOTOR_RATIO * BELT_RATIO ;
//stepper calibs
//1908400 steps (@mstep 8) per turn
// ->  238550 steps for one turn, plus mstepping
float CALIB_STEPS_2_ROT = 238550 * MSTEPPING ;

int SECONDS_PER_DAY = 86400 ;
int speed_3turns1day = 3 * CALIB_STEPS_2_ROT / SECONDS_PER_DAY ; //132,528 @ms16



int stepper_speeds[4] = {MSTEPPING*900, 10000, 1000, 100 } ;



//Pins declaration
int btn_run_pin = 7 ;
int btn_speed_pin[4] = {12, 11, 10, 9} ;
int led_run_pin = 6 ;


//Automaton
Atm_button btn_run;
Atm_button btn_speed[4];
int mode ;
Atm_led led_run;


///////////////   CALLBACKS   //////////////////

void button_change( int idx, int v, int up ){
  mode = idx ;
  Serial.print("Mode ");
  Serial.print(mode); 
  Serial.print("- speed ");
  Serial.println(stepper_speeds[mode]);
}

void run_mode( int idx, int v, int up ){
  if(idx){
    current_speed = stepper_speeds[mode];
    controller.emergencyStop();
    motor.setMaxSpeed(current_speed);
    stepper_mode = true ;
    led_run.trigger(led_run.EVT_BLINK);
  }
  else{
    stepper_mode = false ;
    led_run.trigger(led_run.EVT_OFF);
  }
  

}


///////////////   SETUP   //////////////////

void setup()
{
  delay(2000);
  Serial.begin(115200);
  Serial.println("Started");

  Serial.print("One turn is ");
  Serial.print(STEPS_2_ROT);
  Serial.println(" steps");
  Serial.print("microstepping ");
  Serial.println(MSTEPPING);


  //Automaton setup
  btn_run.begin( btn_run_pin ) 
    // .onPress( [] ( int idx, int v, int up ) {
    //  Serial.println("toggle run");
    //  led_run.trigger(led_run.EVT_TOGGLE);
    // } )
    .onPress( run_mode, 1 )
    .onRelease(run_mode, 0);
  
  led_run.begin(led_run_pin);


  
  for(int i=0; i<4; i++)  {
      btn_speed[i].begin(btn_speed_pin[i])
      .onPress(button_change, i)
      .onRelease(button_change, 0);
    }


  //Automaton.debug
  if(debug){
    btn_run.trace(Serial);
    led_run.trace(Serial);
    for(int i=0; i<4; i++)  {
      btn_speed[i].trace(Serial);
    }
  }

  //Motor setup
  pinMode(enablePin, OUTPUT);
  digitalWrite(enablePin, LOW);
  motor.setMaxSpeed(stepper_maxSpeed);
  motor.setAcceleration(stepper_maxAcc);
}

/////////////////////   LOOP    /////////////////////

void loop() 
{
  automaton.run();
  if(stepper_mode){controller.rotateAsync(motor);}
  else{controller.stopAsync();}
}